#!/usr/bin/perl
###############################################################################
#
#	Funktion	Kovertiert WAV-Dateien mit Programmen des LC-80 ins binre Format 
#
#	Historie:	16.01.2002	by Volker Pohlers created
#			17.08.03 	binmode bei BIN-file ergnzt
#
#	Stand:		$Id$
#
# $Log:$
#
###############################################################################

die <<"help" unless @ARGV;
konvertiert wav-Dateien des LC-80 in Code
Wave-Datei: 8bit, 22khz, mono
help

open IN, $ARGV[0];
binmode IN;
seek IN, 54, SEEK_CUR;	# Wave-Header berspringen

$bufsize = 8*2**10;
$pos = $bufsize;

sub ReadB {
	if ($pos == $bufsize) {
		$bufsize = read(IN, $buf, $bufsize);
		$i++;
		$pos = 0;
	}
	$b = ord(substr($buf, $pos++, 1))-128;
	return $b;
}

# ein Flankenwechsel abwarten (1/2 Periode)
sub ReadSwing1 {
	$w = 1;
	if (ReadB() < 0)	{ while (ReadB() < 0)  { $w++ } }
	else			{ while (ReadB() >= 0) { $w++ } }
	return $w;
}

# eine volle Periode
sub ReadSwing2 {
	$ww = ReadSwing1() + ReadSwing1();
	return $ww;
}


#### LC80
# Wave-Datei: 8bit, 22khz, mono
# -> 1khz entspricht 22 vollen Schwingungen
#    2khz entsprechen 11 vollen Schwingungen

# Test, ob 2kHz oder 1kHz -Periode   
sub Period { # 1 - 2khz, 0 - 1khz
	$p = ReadSwing2();
	if ($p < 15) {return 1} else {return 0};
}

# ein Bit lesen
sub GetBit {
	$h = 0; $l = 0;
	count: {
		if ( !Period() ) {
			$l -= 2;
			$h = 1;
			goto count;
		} else {
			$l++;
			if ($h!=1) { goto count };
		}
	} return $l<=0?1:0;
}

# ein Byte lesen
sub GetByte {
	$e = 0;
	GetBit();
	for ($d=0;$d<8;$d++) {
		$e += 2**$d if GetBit();
	}
	GetBit();
	return $e;	
}

#------------------------------------------------------------------------------
# Laderoutine
#------------------------------------------------------------------------------

# Anfang suchen (1000x 1 kHz)
LEAD:
	$hl = 1000;
LEAD1:
	goto LEAD if Period() == 1;
	$hl--;
	goto LEAD1 if $hl;

# restliche 1KHz Tne berspringen
while (Period() == 0) {}

# Kopfblock
$filename = GetByte() + 256 * GetByte();
$startadr = GetByte() + 256 * GetByte();
$endadr = GetByte() + 256 * GetByte();
$chksum = GetByte();

$binfilename = sprintf "%.4lX (%.4lX-%.4lX).bin", $filename, $startadr, $endadr;

open BIN, ">$binfilename.bin";
open MC, ">$binfilename.mc";

binmode BIN;

printf MC "Filename: %.4lX\n", $filename;
printf MC "Startadr: %.4lX\n", $startadr;
printf MC "Endadr:   %.4lX\n", $endadr;
printf MC "Chksum:   %.2lX\n", $chksum;

# 2 kHz Trennblock berlesen
while (Period() == 1) {}
	
# Bytes einlesen	
$per = 0;	
$mychksum = 0;
for (my $i=0;$i<$endadr-$startadr; $i++) {
	my $byte = GetByte();
	printf MC "%.2lX ", $byte; $per++; if ($per == 16) {print MC "\n"; $per = 0}
	$mychksum += $byte;
	syswrite BIN, pack ("C", $byte);
}
$mychksum %= 256;

if ($mychksum == $chksum) {
	print MC "\nok\n";
} else {
	printf MC "\nChksum-Error (sum %.2lX)\n", $mychksum;
}
	
close IN;
close MC;
close BIN;

print "ok\n";
__END__

