; Programm
; 2716 (EPROM-Programmierung)
; Autor: Gunther Zielosko
; "Hinweise zur Anwendung des Lerncomputers LC 80", Teil 2, Seite 17-28
; korr. 01.04.2013 
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

	CPU	Z80

	INCLUDE	lc80

;	ORG	1000h
	ORG	2000h

; Hauptprogramm

; Das Hauptprogramm organisiert den gesamten Programmierablauf einschlielich der
; Funktionen:
; - Leerkontrolle (Ist EPROM vollstndig leer? - FF?)
; - Einlesen eines programmierten EPROMs in den RAM-Bereich
; - Vergleich Ist- und Sollwert jeder Zelle.


HAUPT:	LD	HL, memad
	LD	DE, 23EBh
	LD	BC, 0006h
	LDIR
	CALL	INIT		;Initialisierung Port B
H1:	CALL	AUS		;Grundzustand: VCC,VPP = 0 V
	LD	IX, aU2716	;Anzeige "U 2716"
	CALL	DAK1
	CP	A, 1Fh		;LD ?
	PUSH	AF
	CALL	Z, LESEN
	POP	AF
	CP	A, 1Eh		;ST ?
	JR	NZ, H1
	CALL	LEER
	LD	B, 05h		;5 Programmierzyklen
H2:	PUSH	BC
	CALL	PROGR
	CALL	SOU1K+1		;1 Ton aus dem Monitorprogramm
	CALL	VERGL
	POP	BC
	DJNZ	H2
	PUSH	AF
	CALL	AUS
	POP	AF
	JR	C, H3
H4:	LD	IX, TXT_ERR	;Anzeige "ERROR"
	CALL	DAK1
	JR	H1
H3:	LD	IX, aREADY	;Anzeige "READY"
	CALL	DAK1
	JR	H1

; Unterprogramm INIT

; Das Unterprogramm INIT dient zur Initialisierung des Port B der USER-PIO.

INIT:	LD	A, 0FFh		;PIO-Port B, Mode 3
	OUT	PIO2BC, A
	LD	A, 0F0h 	;E/A-Definition:
	OUT	PIO2BC, A	;B 0 ... B 3 = Ausgnge, B 4 ... B 7 = Eingnge
	RET
	NOP

; Unterprogramm LESEN

; Das Unterprogramm LESEN realisiert die bernahme des Inhaltes des auf der
; Programmierfassung steckenden EPROMs U 2716 C in den RAM-Speicher des LC-80 (2.
; und 3. RAM, Adressen 2400 bis 2BFF). Nach Abschlu des Vorganges erscheint die
; Anzeige "READY".

LESEN:	CALL	ADVOR
L1:	CALL	ADINK
	CALL	DATIN
	LD	(IY+00h), A
	INC	IY
	CALL	ENDE
	JR	NZ, L1
	LD	IX, aREADY	;Anzeige "READY"
	CALL	DAK1
	RET

; Unterprogramm LEER

; Das Unterprogramm LEER vergleicht einzeln jedes Byte des auf der
; Programmierfassung steckenden EPROMs mit FF. Sind alle Zellen leer, wird "LEER"
; angezeigt, ansonsten "ERROR".


LEER:	CALL	ADVOR
O1:	CALL	ADINK
	CALL	DATIN
	CP	A, 0FFh		;Vergleich mit FF
	JR	NZ, O2
	CALL	ENDE
	JR	NZ, O1
	LD	IX, aLEER	;Anzeige "LEER"
	LD	B, 0FFh		;Anzeigetest
O3:	CALL	DAK2
	DJNZ	O3
	RET
O2:	LD	HL, H4
	EX	(SP), HL
	RET

; Unterprogramm PROGR

; Im Unterprogramm PROGR wird der Inhalt von RAM 2 und 3 byteweise in den auf der
; Programmierfassung steckenden EPROM geladen. Nach Abschlu erfolgt die Anzeige
; "READY". Pro Byte wird ein Programmierimpuls von ca. 10 ms erzeugt. Diese Zeit
; wird jedesmal durch Aufruf von DAK2 realisiert, ein Nebeneffekt ist die Anzeige
; von Adressen und Daten auf dem Display.

PROGR:	LD	IX, 23F2h
	CALL	ADVOR
	IN	A, 0F9h
	SET	3, A		;/OE => 1   Programmier-
	RES	2, A		;/CE => 0   sperre
	OUT	0F9h, A
P1:	CALL	ADINK
	LD	A, (IY+00h)
	OUT	0F8h, A
	CALL	DADP
	LD	HL, 23F8h
	LD	E, (HL)
	INC	HL
	LD	D, (HL)
	DEC	DE
	CALL	ADRSDP
	IN	A, 0F9h
	SET	2, A		;/CE => 1
	OUT	0F9h, A
	CALL	DAK2		;Programmierimpuls ca. 10 ms
	IN	A, 0F9h
	RES	2, A		;/CE => 0
	OUT	0F9h, A
	INC	IY
	CALL	ENDE
	JR	NZ, P1
	RET

; Unterprogramm VERGL

; Das Unterprogramm VERGL organisiert einen Bitmustervergleich zwischen EPROM und
; Zusatz-RAM. Bei bereinstimmung erscheint "PASS", bei Nichtbereinstimmung
; "FAIL".

VERGL:	CALL	ADVOR
V1:	CALL	ADINK
	CALL	DATIN
	LD	B, (IY+00h)
	INC	IY
	CP	A, B
	JR	NZ, V2
	CALL	ENDE
	JR	NZ, V1
	LD	IX, aPass-1	;Anzeige "PASS"
	SCF
	JR	V3
V2:	LD	IX, aFail	;Anzeige "FAIL"
	OR	A		;CY: = 0
V3:	LD	B, 0FFh		;Anzeigezeit
V4:	PUSH	AF
	CALL	DAK2
	POP	AF
	DJNZ	V4
	RET

; Unterprogramm ADVOR

; ADVOR dient zur Herstellung der Anfangsbedingungen bei der Adressierung von RAM
; und EPROM sowie zur Nullsetzung des Byte-Zhlers.

ADVOR:	LD	HL, 23FEh
	LD	(HL), 00h
	INC	HL
	LD	(HL), 00h
	LD	HL, 23EDh
	LD	E, (HL)
	INC	HL
	LD	D, (HL)
	PUSH	DE
	POP	IY
	INC	HL
	LD	DE, 23F8h
	LD	BC, 0002h
	LDIR
	LD	A, 07h
	OUT	0F9h, A
	RET

; Unterprogramm ADINK

; Das Unterprogramm ADINK organisiert die Erhhung der Adressen von RAM und EPROM
; sowie des Byte-Zhlers, bis der vorgegebene Betrag (2 KByte) erreicht wird.

ADINK:	LD	A, 0Fh
	OUT	0FAh, A
	LD	HL, 23F8h
ADNK1:	IN	A, 0F9h		;Im Unterprogramm AUS wird zu dieser Marke gesprungen.
	RES	0, A
	OUT	0F9h, A
	LD	A, (HL)
	LD	C, A
	OUT	0F8h, A
	INC	HL
	IN	A, 0F9h
	SET	0, A
	RES	1, A
	OUT	0F9h, A
	LD	A, (HL)
	LD	B, A
	OUT	0F8h, A
	INC	BC
	LD	(HL), B
	DEC	HL
	LD	(HL), C
	IN	A, 0F9h
	SET	1, A
	OUT	0F9h, A
	LD	HL, 23FEh
	LD	E, (HL)
	INC	HL
	LD	D, (HL)
	INC	DE
	LD	(HL), D
	DEC	HL
	LD	(HL), E
	RET

; Unterprogramm DATIN

; Das Unterprogramm DATIN ist zustndig fr das Einlesen eines EPROM-Bytes in das
; entsprechende RAM-Byte bei vorher organisierter Adressierung.

DATIN:	IN	A, 0F9h
	RES	2, A
	OUT	0F9h, A
	LD	A, 4Fh
	OUT	0FAh, A
	IN	A, 0F8h
	PUSH	AF
	IN	A, 0F9h
	SET	2, A
	OUT	0F9h, A
	POP	AF
	RET

; Unterprogramm AUS

; Bei AUS wird auf allen PIO-Leitungen 0 ausgegeben, damit werden auch UCC und UPR
; abgeschaltet, ebenso /CE und /OE.

AUS:	LD	A, 0Fh
	OUT	0FAh, A
	LD	HL, 098Ah-ROMOFS	;Bei Bestckung des LC-80 mit 2 ROMs U 505 D ist hier einzutragen: LD HL, 098A
	CALL	ADNK1		;Hier wird nicht ADINK angesprungen, sondern die Marke 1 in diesem Unterprogramm!
	LD	A, 00h
	OUT	0F9h, A
	RET

; Unterprogramm ENDE

; Das Unterprogramm ENDE vergleicht den Byte-Zhler mit 2 K (hexadezimal 800). Je
; nach Ergebnis wird das Zero-Flag gesetzt. Bei Z = 1 sind alle Bytes
; abgearbeitet.

ENDE:	LD	HL, 23FEh
	LD	C, (HL)
	INC	HL
	LD	B, (HL)
	LD	HL, 23EBh
	LD	E, (HL)
	INC	HL
	LD	D, (HL)
	EX	DE, HL
	OR	A			; CY = 0 setzen, Z-Flag setzen, wenn H1 = BC
	SBC	HL, BC
	RET

; Tabelle der Anzeigetexte und Ladeparameter

memad:	DW	00800h			;Anzahl der zu bertragenden Bytes (800 hexadezimal - 2048 dezimal - 2 KByte)
	DW	02400h			;Anfangsadresse im RAM-Bereich (2400)
	DW	00000h			;Anfangsadresse beim zu programmierenden EPROM U 2716 C (0000)

aPASS:	DB	0AEh			;"S"
	DB	0AEh			;"S"		"PASS"
	DB	06Fh			;"A"
	DB	04Fh			;"P"
	DB	000h

aREADY:	DB	000h
	DB	0ABh			;"Y"
	DB	0E9h			;"D"
	DB	06Fh			;"A"		"READY"
	DB	0CEh			;"E"
	DB	048h			;"R"

	DB	000h

aLEER:	DB	000h
	DB	048h			;"R"
	DB	0CEh			;"E"		"LEER"
	DB	0CEh			;"E"
	DB	0C2h			;"L"
	DB	000h

aU2716:	DB	0EEh			;"6"
	DB	021h			;"1"
	DB	025h			;"7"		"U 2716"
	DB	0CDh			;"2"
	DB	000h
	DB	0E3h			;"U"

aFAIL:	DB	000h
	DB	0C2h			;"L"
	DB	021h			;"I"		"FAIL"
	DB	06Fh			;"A"
	DB	04Eh			;"F"
	DB	000h

	DB	0FFh

	END
